clear all
close all
clc


% CAN, FRA, DEU, ITA, JPN, GBR, USA

tab.NGDP_y = readtable('data\G7_final_revision.xlsx','ReadRowNames',true,'Sheet','(y) Nominal GDP');
tab.CPI_y = readtable('data\G7_final_revision.xlsx','ReadRowNames',true,'Sheet','(y) CPI');
tab.TCREDIT_y = readtable('data\G7_final_revision.xlsx','ReadRowNames',true,'Sheet','(y) Total loans');

tab.RGDP_q = readtable('data\G7_final_revision.xlsx','ReadRowNames',true,'Sheet','(q) RGDP');
tab.CQ_q = readtable('data\G7_final_revision.xlsx','ReadRowNames',true,'Sheet','(q) CQ-Ratio');

%Fill missing with linear interpolation 
tab.TCREDIT_y{69:77,2} = interp1([69 77],tab.TCREDIT_y{[69 77],2}',[69:77],'linear')'; %Checked
tab.TCREDIT_y{71:77,3} = interp1([71 77],tab.TCREDIT_y{[71 77],3}',[71:77],'linear')'; %Checked
tab.TCREDIT_y{51:55,3} = interp1([51 55],tab.TCREDIT_y{[51 55],3}',[51:55],'linear')'; %Checked

tab.NGDP_y{75:77,3} = interp1([75 77],tab.NGDP_y{[75 77],3}',[75:77],'linear')'; %Checked
tab.NGDP_y{75:77,5} = interp1([75 77],tab.NGDP_y{[75 77],5}',[75:77],'linear')'; %Checked
tab.NGDP_y{51:55,3} = interp1([51 55],tab.NGDP_y{[51 55],3}',[51:55],'linear')'; %Checked

tab.CPI_y{51:55,3} = interp1([51 55],tab.CPI_y{[51 55],3}',[51:55],'linear')'; %Checked



%Yearly: Joint sample is sample in paper
tab.RGDP_y = log(tab.NGDP_y{:,:}./tab.CPI_y{:,:})*100;
tab.CQ_y = 100*tab.TCREDIT_y{:,:}./tab.NGDP_y{:,:};



%Quarterly: Cut as in paper
%keep sample per country align, to make comparison of standard deviations
tab.RGDP_q = log(tab.RGDP_q{9:290,:})*100; %1952Q1:2022Q2
tab.CQ_q = tab.CQ_q{9:290,:};

tab.RGDP_y = tab.RGDP_y(11:end,:); %1880:2020
tab.CQ_y = tab.CQ_y(11:end,:);

save.time_q = [1952.25 : 0.25 : 2022.5]';
save.time_y = [1880:1:2020]';

load lams_new

save_std_RGDP_y = [];
save_std_RGDP_q = [];
save_std_CQ_y = [];
save_std_CQ_q = [];

save_pers_RGDP_y = [];
save_pers_RGDP_q = [];
save_pers_CQ_y = [];
save_pers_CQ_q = [];

save_rmse_RGDP_y = [];
save_rmse_RGDP_q = [];
save_rmse_CQ_y = [];
save_rmse_CQ_q = [];


save_corr_RGDP_y = [];
save_corr_RGDP_q = [];
save_corr_CQ_y = [];
save_corr_CQ_q = [];

for ii = 1:2 %loop across yearly, then quarterly
    
    if ii == 1 %yearly
        
        y = tab.RGDP_y;
        c = tab.CQ_y;
        
        lam = I(1:2,1);
        lam_star = I(1:2,2);
        sig = sqrt(I(1:2,3));
        
    elseif ii==2

        y = tab.RGDP_q;
        c = tab.CQ_q;
        
        lam = I(3:4,1);
        lam_star = I(3:4,2);
        sig = sqrt(I(3:4,3));

        
    end
    
    for jj = 1:7% loop across G7 countries
        
        temp_y = y(:,jj);
        temp_c = c(:,jj);
        
        save_gdp = nan(length(temp_y),3);
        save_cgdp = nan(length(temp_c),3);
        
        
        idx = ~isnan(temp_y.*temp_c);
        
        temp_y_idx = temp_y(idx,1);
        temp_c_idx = temp_c(idx,1);
        
        %produce cyclical component
         [~,y_cyc_2s] = hpfilter(temp_y_idx,lam(1));
         [~,y_cyc_1s_star] = fn_hpfilter_rt(temp_y_idx,lam_star(1));
         y_cyc_1s_star =  y_cyc_1s_star*sig(1);
         [~,y_cyc_1s] = fn_hpfilter_rt(temp_y_idx,lam(1));
        
         [~,c_cyc_2s] = hpfilter(temp_c_idx,lam(2));
         [~,c_cyc_1s_star] = fn_hpfilter_rt(temp_c_idx,lam_star(2));
         c_cyc_1s_star =  c_cyc_1s_star*sig(2);
         [~,c_cyc_1s] = fn_hpfilter_rt(temp_c_idx,lam(2));
         
         save_gdp(idx,:) = [y_cyc_2s y_cyc_1s_star y_cyc_1s];
         save_cgdp(idx,:) = [c_cyc_2s c_cyc_1s_star c_cyc_1s];
      
         %get stats
         
             [b1_y, ~, ~, ~, ~, ~, ~, ~] = olsnw(y_cyc_2s(2:end,:),y_cyc_2s(1:end-1,:),1);
             [b2_y, ~, ~, ~, ~, ~, ~, ~] = olsnw(y_cyc_1s_star(2:end,:),y_cyc_1s_star(1:end-1,:),1);
             [b3_y, ~, ~, ~, ~, ~, ~, ~] = olsnw(y_cyc_1s(2:end,:),y_cyc_1s(1:end-1,:),1);

             [b1_c, ~, ~, ~, ~, ~, ~, ~] = olsnw(c_cyc_2s(2:end,:),c_cyc_2s(1:end-1,:),1);
             [b2_c, ~, ~, ~, ~, ~, ~, ~] = olsnw(c_cyc_1s_star(2:end,:),c_cyc_1s_star(1:end-1,:),1);
             [b3_c, ~, ~, ~, ~, ~, ~, ~] = olsnw(c_cyc_1s(2:end,:),c_cyc_1s(1:end-1,:),1);
            

         
         
       if ii == 1
               
        save_std_RGDP_y = [save_std_RGDP_y std([y_cyc_2s y_cyc_1s_star y_cyc_1s])'];
        save_std_CQ_y = [save_std_CQ_y std([c_cyc_2s c_cyc_1s_star c_cyc_1s])'];

        save_pers_RGDP_y = [save_pers_RGDP_y [b1_y(2) b2_y(2) b3_y(2)]'];
        save_pers_CQ_y = [save_pers_CQ_y [b1_c(2) b2_c(2) b3_c(2)]'];

        save_rmse_RGDP_y = [save_rmse_RGDP_y [sqrt(mean((y_cyc_2s-y_cyc_1s_star).^2))/sqrt(mean((y_cyc_2s-y_cyc_1s).^2))]'];
        save_rmse_CQ_y = [save_rmse_CQ_y [sqrt(mean((c_cyc_2s-c_cyc_1s_star).^2))/sqrt(mean((c_cyc_2s-c_cyc_1s).^2))]'];

        save_corr_RGDP_y = [save_corr_RGDP_y [corr(y_cyc_2s,y_cyc_1s_star) corr(y_cyc_2s,y_cyc_1s)]'];
        save_corr_CQ_y = [save_corr_CQ_y [corr(c_cyc_2s,c_cyc_1s_star) corr(c_cyc_2s,c_cyc_1s)]'];

        
        figname_gdp = ['GDPy_all_' num2str(jj)];
        figname_cgdp = ['cGDPy_all_' num2str(jj)];

        
        fn_plot3(save_gdp,save.time_y,'HP-2s','HP-1s*', 'HP-1s',figname_gdp,[0 0 0],[0 0 1],[1 0 0]);
        fn_plot3(save_cgdp,save.time_y,'HP-2s','HP-1s*', 'HP-1s',figname_cgdp,[0 0 0],[0 0 1],[1 0 0]);
        if jj == 7
        fn_plot3_legend(save_cgdp,save.time_y,'HP-2s','HP-1s*', 'HP-1s',figname_cgdp,[0 0 0],[0 0 1],[1 0 0]);
        end

        elseif ii == 2
        
        save_std_RGDP_q = [save_std_RGDP_q std([y_cyc_2s y_cyc_1s_star y_cyc_1s])'];
        save_std_CQ_q = [save_std_CQ_q std([c_cyc_2s c_cyc_1s_star c_cyc_1s])'];
        
        save_pers_RGDP_q = [save_pers_RGDP_q [b1_y(2) b2_y(2) b3_y(2)]'];
        save_pers_CQ_q = [save_pers_CQ_q [b1_c(2) b2_c(2) b3_c(2)]'];

        save_rmse_RGDP_q = [save_rmse_RGDP_q [sqrt(mean((y_cyc_2s-y_cyc_1s_star).^2))/sqrt(mean((y_cyc_2s-y_cyc_1s).^2))]'];
        save_rmse_CQ_q = [save_rmse_CQ_q [sqrt(mean((c_cyc_2s-c_cyc_1s_star).^2))/sqrt(mean((c_cyc_2s-c_cyc_1s).^2))]'];

        save_corr_RGDP_q = [save_corr_RGDP_q [corr(y_cyc_2s,y_cyc_1s_star) corr(y_cyc_2s,y_cyc_1s)]'];
        save_corr_CQ_q = [save_corr_CQ_q [corr(c_cyc_2s,c_cyc_1s_star) corr(c_cyc_2s,c_cyc_1s)]'];


        figname_gdp = ['GDPq_all_' num2str(jj)];
        figname_cgdp = ['cGDPq_all_' num2str(jj)];

        fn_plot3(save_gdp,save.time_q,'HP-2s','HP-1s*', 'HP-1s',figname_gdp,[0 0 0],[0 0 1],[1 0 0]);
        fn_plot3(save_cgdp,save.time_q,'HP-2s','HP-1s*', 'HP-1s',figname_cgdp,[0 0 0],[0 0 1],[1 0 0]);
        if jj == 7
        fn_plot3_legend(save_cgdp,save.time_q,'HP-2s','HP-1s*', 'HP-1s',figname_cgdp,[0 0 0],[0 0 1],[1 0 0]);
        end
     
       
       end
    end
end
        

Table2 = [(save_std_CQ_y-save_std_RGDP_y)./save_std_RGDP_y;...
         (save_std_CQ_q-save_std_RGDP_q)./save_std_RGDP_q;...
         (save_pers_CQ_y-save_pers_RGDP_y);...
         (save_pers_CQ_q-save_pers_RGDP_q)];
        
        
        