# -*- coding: utf-8 -*-
"""
Created on Tue Jan 12 00:22:58 2021

@author: eliaswolf
"""

import numpy as np
import pandas as pd
import statsmodels.api as sm
import os as os

# Set path to folder with summary_functions
path = r'...'
os.chdir(path)

import summary_functions as sry

from tqdm import tqdm
from statsmodels.tsa.stattools import acf


'''
Code for the Simulations of the paper "On Adjusting the One-Sided HP-Filter" 
by Elias Wolf, Frieder Mokinsiki and Yves Schüler. This Code replicates 
the results in Table 1 (Sample Size 100) for a White Noise Process a the Random Walk. 
The results are printed in the console. Results from the Online Appendix 
for smaller and larger sample sizes (T=50, 500, 1000) are also shown. 

For further questions or remaing errors please contact: e.wolf@uni-bonn.de

'''

# Simulations
N_Sim = 1000

# List with sample sizes
N_list = [50, 100, 500, 1000]

# Simulations are stored here

sigma_all = []  # Standard Deviations
rho_all = []  # Mean Autocorrelation 
acf_all = []  # All Autocorrelations

# Outer Loop over different sample sizes
for N in N_list:
    
    print("Number of Obs: " + str(N))
    
    lambda_OSTS = [6.25, 1600, 1562.25, 400000]
    lambda_adj = [2.45, 650, 634.84, 163101]
    kappa = [1.7962, 1.1513, 1.1519, 1.0360]
    
    burn_in = 20
    N_obs = N + burn_in
    
    sigma = []
    rho = []
    ac_func = []
    
    # First inner Loop over different target of frequency bands
    for kap, lam, lam_adj in zip(kappa, lambda_OSTS, lambda_adj):
        
        sd_OS = [[] for i in range(3)]
        sd_adj = [[] for i in range(3)]
        sd_TS = [[] for i in range(3)]
        
        acorr_OS = [[] for i in range(3)]
        acorr_adj = [[] for i in range(3)]
        acorr_TS = [[] for i in range(3)]  
        
        acorr_all_OS = [[] for i in range(3)]
        acorr_all_adj = [[] for i in range(3)]
        acorr_all_TS = [[] for i in range(3)]
        
        if lam == 6.25:
                    
            param = "6_25"
        
        if lam == 1600:
            
            param = "1600"
            
        if lam == 1562.25:
            
            param = "1562_25"
            
        if lam == 400000:
            
            param = "400000"
        
        # Second inner loop over Sample Size
        for i in tqdm(range(N_Sim)):
            
            # Simulate Data
            
            # White Noise
            noise = np.random.normal(0, 1, size=N_obs)
            
            # Random Walk
            random_walk = np.cumsum(noise)
            
            # Collect in Data Frame
            process = pd.DataFrame({"noise":noise,
                                    "random_walk":random_walk})
            
            # Third Inner Loop over Processes
            for j in range(2):
                
                # Pick Process
                us_gdp = process.iloc[:,j]
                
                # Two-Sided Filter
                cycle_TS, trend_TS = sm.tsa.filters.hpfilter(us_gdp[burn_in:], lam)
        
                filtered_cycles_OS = []
                filtered_cycles_adj = []
        
                # Expanding window filtering One Sided
                for k in range(0, len(us_gdp)-burn_in):
        
                    cycle, trend = sm.tsa.filters.hpfilter(us_gdp[0:burn_in+k], lam)
        
                    filtered_cycles_OS.append(np.asarray(cycle)[-1])
        
                # Expanding window filtering One Sided adjusted
                for k in range(0, len(us_gdp)-burn_in):
        
                    cycle, trend = sm.tsa.filters.hpfilter(us_gdp[0:burn_in+k], lam_adj)
        
                    filtered_cycles_adj.append(np.asarray(cycle)[-1])
                
                # Rescale cyclical Component
                filtered_cycles_adj = kap*np.asarray(filtered_cycles_adj)
                
                # Standard Deviation 
                # Store Autocorrelation for HP-1s, HP-1s* and HP-2s
                acorr_OS[j].append(np.mean(acf(filtered_cycles_OS, nlags=12)[1:]))
                acorr_adj[j].append(np.mean(acf(filtered_cycles_adj, nlags=12)[1:]))
                acorr_TS[j].append(np.mean(acf(cycle_TS, nlags=12)[1:]))
                
                acorr_all_OS[j].append(acf(filtered_cycles_OS, nlags=12)[1:])
                acorr_all_adj[j].append(acf(filtered_cycles_adj, nlags=12)[1:])
                acorr_all_TS[j].append(acf(cycle_TS, nlags=12)[1:])
                
                # Store Standard Deviation for HP-1s, HP-1s* and HP-2s
                sd_OS[j].append(np.std(filtered_cycles_OS))
                sd_adj[j].append(np.std(filtered_cycles_adj))
                sd_TS[j].append(np.std(cycle_TS))
        
        
        sd_noise = pd.DataFrame({"One-Sided": sd_OS[0],
                                 "Adjusted": sd_adj[0],
                                 "Two-Sided": sd_TS[0]})
        
        sd_random = pd.DataFrame({"One-Sided": sd_OS[1],
                                  "Adjusted": sd_adj[1],
                                  "Two-Sided": sd_TS[1]}) 


        
        acorr_noise = pd.DataFrame({"One-Sided": acorr_OS[0],
                                    "Adjusted": acorr_adj[0],
                                    "Two-Sided": acorr_TS[0]})
        
        acorr_random = pd.DataFrame({"One-Sided": acorr_OS[1],
                                     "Adjusted": acorr_adj[1],
                                     "Two-Sided": acorr_TS[1]}) 
        
        acf_arrays_noise = {"acf_OS_" + param + "_"  + str(N): acorr_all_OS[0],
                            "acf_TS_" + param + "_"  + str(N): acorr_all_TS[0],
                            "acf_adj_" + param + "_"  + str(N): acorr_all_adj[0]}
        
        acf_arrays_random = {"acf_OS_" + param + "_"  + str(N): acorr_all_OS[1],
                             "acf_TS_" + param + "_"  + str(N): acorr_all_TS[1],
                             "acf_adj_" + param + "_"  + str(N): acorr_all_adj[1]}

        
        sd = {"noise": sd_noise,
              "random_walk": sd_random}
        
        acorr = {"noise": acorr_noise, 
                 "random_walk": acorr_random}
        
        acf_arrays = {"noise": acf_arrays_noise,
                      "random_walk": acf_arrays_random}    
        
        sigma.append(sd)
        rho.append(acorr)
        ac_func.append(acf_arrays)
        
    sigma_all.append(sigma)
    rho_all.append(rho)
    acf_all.append(ac_func)


# Collect results in dictionaries as input to summary functions
simulated_sigmas = {"50": sigma_all[0],
                    "100": sigma_all[1],
                    "500": sigma_all[2],
                    "1000": sigma_all[3]}
    
simulated_rhos = {"50": rho_all[0],
                  "100": rho_all[1],
                  "500": rho_all[2],
                  "1000": rho_all[3]}

simulated_acf = {"50": acf_all[0],
                 "100": acf_all[1],
                 "500": acf_all[2],
                 "1000": acf_all[3]}
    
   
'''
Print Rows 1 - 6 (White Noise and Random Walk) for Columns 1 and 2 
(relative difference of standard deviations and absolute difference 
of autocorrelations) from Table 1 in the main paper. 
'''

# Print results for standard deviations
print("\n T=100: -------------- White Noise (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 100, [6.25, 1600, 400000], "noise")
print("\n -------------- Random Walk (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 100, [6.25, 1600, 400000], "random_walk")



# Print results for autocorrelations
print("\n -------------- White Noise (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 100, [6.25, 1600, 400000], "noise")
print("\n -------------- Random Walk (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 100, [6.25, 1600, 400000], "random_walk")


###############################################################################
"""
Results from the Online Appendix for Sample Sizes 50, 500, 1000
"""

# Sample Size 50

print("\n T=50: \n -------------- White Noise (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 50, [6.25, 1600, 400000], "noise")
print("\n -------------- Random Walk (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 50, [6.25, 1600, 400000], "random_walk")


# Print results for autocorrelations
print("\n -------------- White Noise (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 50, [6.25, 1600, 400000], "noise")
print("\n -------------- Random Walk (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 50, [6.25, 1600, 400000], "random_walk")

###############################################################################

# Sample Size 500

print("\n T=500: \n -------------- White Noise (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 500, [6.25, 1600, 400000], "noise")
print("\n -------------- Random Walk (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 500, [6.25, 1600, 400000], "random_walk")


# Print results for autocorrelations
print("\n -------------- White Noise (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 500, [6.25, 1600, 400000], "noise")
print("\n -------------- Random Walk (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 500, [6.25, 1600, 400000], "random_walk")

###############################################################################

# Sample Size 1000

print("\n T=1000: \n -------------- White Noise (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 1000, [6.25, 1600, 400000], "noise")
print("\n -------------- Random Walk (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 1000, [6.25, 1600, 400000], "random_walk")


# Print results for autocorrelations
print("\n -------------- White Noise (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 1000, [6.25, 1600, 400000], "noise")
print("\n -------------- Random Walk (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 1000, [6.25, 1600, 400000], "random_walk")