# -*- coding: utf-8 -*-
"""
Created on Tue Apr  4 17:34:56 2023

@author: elias
"""

import numpy as np
import pandas as pd
import os as os

# Set path to folder with file summary_functions and data
path = r'...'
os.chdir(path)

import summary_functions as sry
import statsmodels.api as sm
import matplotlib.pyplot as plt
from tqdm import tqdm
from statsmodels.tsa.stattools import acf
from statsmodels.tsa.arima.model import ARIMA
from statsmodels.tsa.stattools import adfuller

'''
Code for the Simulations of the paper "On Adjusting the One-Sided HP-Filter" 
by Elias Wolf, Frieder Mokinsiki and Yves Schüler. This Code replicates 
the results in Table 1 (Sample Size 100) for a White Noise Process a the Random Walk. The results 
are printed in the console. Results from the Online Appendix for smaller and 
larger sample sizes (T=50, 500, 1000) are also shown. 

For further questions or remaing errors please contact: e.wolf@uni-bonn.de

'''


# Function to print output of ADF-Test
def adf_test(timeseries, reg_type):
    print ('Results of Dickey-Fuller Test:')
    dftest = adfuller(timeseries, autolag='AIC', regression=reg_type)
    dfoutput = pd.Series(dftest[0:4], index=['Test Statistic','p-value','#Lags Used','Number of Observations Used'])
    for key,value in dftest[4].items():
       dfoutput['Critical Value (%s)'%key] = value
    print (dfoutput)


# Quaterly 

data_yr = pd.read_excel("SIM_DataForARIMA.xlsx", sheet_name=0)
data_qr = pd.read_excel("SIM_DataForARIMA.xlsx", sheet_name=1)

# GDP Quarterly 
data_us_quarterly = data_qr.iloc[:271,1:]
data_us_quarterly.index = pd.period_range('1952Q2', '2019Q4', freq='Q')


plt.plot(np.asarray(data_us_quarterly["LOG rgdp"])/100)
plt.grid()
plt.show()

growth_gdp_usq = data_us_quarterly["LOG rgdp"].diff().dropna()


plt.plot(np.asarray(growth_gdp_usq))
plt.grid()
plt.show()

adf_test(growth_gdp_usq, "c")

order_grid = []

for i in range(1, 4):
    for j in range(1):
        for k in range(1, 4):
            order_grid.append(tuple([i, j, k]))
   
inf_crit_gdpq = []
        
for order in order_grid:
    
    print(order)
    
    ARIMA_object = ARIMA(endog = growth_gdp_usq, order=order, trend='c')
    ARIMA_est = ARIMA_object.fit(method='innovations_mle')
    inf_crit_gdpq.append(ARIMA_est.aic)

order = order_grid[np.argmin(inf_crit_gdpq)]

ARIMA_gdpq = ARIMA(endog = growth_gdp_usq, order=order, trend='c')
ARIMA_est_gdpq = ARIMA_gdpq.fit(method='innovations_mle')
print(ARIMA_est_gdpq.summary())



# Credit to GDP Quaterly 

plt.plot(np.asarray(data_us_quarterly["credit-to-gdp"]))
plt.grid()
plt.show()

growth_cgdp_usq = data_us_quarterly["credit-to-gdp"].diff().dropna()


plt.plot(np.asarray(growth_cgdp_usq))
plt.grid()
plt.show()

adf_test(growth_cgdp_usq, "c")

order_grid = []

for i in range(1, 4):
    for j in range(1):
        for k in range(0, 4):
            order_grid.append(tuple([i, j, k]))
   
inf_crit_cgdpq = []
        
for order in order_grid:
    
    print(order)
    
    ARIMA_object = ARIMA(endog = growth_cgdp_usq, order=order, trend='c')
    ARIMA_est = ARIMA_object.fit(method='innovations_mle')
    inf_crit_cgdpq.append(ARIMA_est.aic)

order = order_grid[np.argmin(inf_crit_cgdpq)]

ARIMA_cgdpq = ARIMA(endog = growth_cgdp_usq, order=order, trend='c')
ARIMA_est_cgdpq = ARIMA_cgdpq.fit(method='innovations_mle')
print(ARIMA_est_cgdpq.summary())



# Yearly

data_us_yearly = data_yr.iloc[:140,1:]
data_us_yearly.index = pd.period_range('1880', '2019', freq='Y')

# GDP Yearly

growth_gdp_usy = data_us_yearly["LOG rgdp"].diff().dropna()

plt.plot(np.asarray(growth_gdp_usy))
plt.grid()
plt.show()

adf_test(growth_gdp_usy, "c")

order_grid = []

for i in range(1, 4):
    for j in range(1):
        for k in range(1, 4):
            order_grid.append(tuple([i, j, k]))
   
inf_crit_gdpy = []
        
for order in order_grid:
    
    print(order)
    
    ARIMA_object = ARIMA(endog = growth_gdp_usy, order=order, trend='c')
    ARIMA_est = ARIMA_object.fit(method='innovations_mle')
    inf_crit_gdpy.append(ARIMA_est.aic)

order = order_grid[np.argmin(inf_crit_gdpy)]

ARIMA_gdpy = ARIMA(endog = growth_gdp_usy, order=order, trend='c')
ARIMA_est_gdpy = ARIMA_gdpy.fit(method='innovations_mle')
print(ARIMA_est_gdpy.summary())


# Credit to Gdp Yearly

growth_cgdp_usy = data_us_yearly["credit-to-gdp"].diff().dropna()

plt.plot(np.asarray(growth_cgdp_usy))
plt.grid()
plt.show()

adf_test(growth_cgdp_usy, "c")

inf_crit_cgdpy = []
        
for order in order_grid:
    
    print(order)
    
    ARIMA_object = ARIMA(endog = growth_cgdp_usy, order=order, trend='c')
    ARIMA_est = ARIMA_object.fit(method='innovations_mle')
    inf_crit_cgdpy.append(ARIMA_est.aic)

order = order_grid[np.argmin(inf_crit_cgdpy)]

ARIMA_cgdpy = ARIMA(endog = growth_cgdp_usy, order=order)
ARIMA_est_cgdpy = ARIMA_cgdpy.fit(method='innovations_mle')
print(ARIMA_est_cgdpy.summary())


# Simulations
N_Sim = 1000

# List with sample sizes
N_list = [50, 100, 500, 1000]

# Simulations are stored here
sigma_all = []  # Standard Deviations
rho_all = []  # Autocorrelation 1
acf_all = []


# Outer Loop over different sample sizes
for N in N_list:
    
    print("Number of Obs: " + str(N))
    
    lambda_OSTS = [6.25, 1600, 1562.25, 400000]
    lambda_adj = [2.45, 650, 634.84, 163101]
    kappa = [1.7962, 1.1513, 1.1519, 1.0360]
    
    burn_in = 20
    N_obs = N + burn_in
    
    sigma = []
    rho = []
    acfs = []
    
    # First inner Loop over different target of frequency bands
    for kap, lam, lam_adj in zip(kappa, lambda_OSTS, lambda_adj):
        
        sd_OS = []
        sd_adj = []
        sd_TS = []
        
        acorr_OS = []
        acorr_adj = []
        acorr_TS = []  
        
        acorr_all_OS = []
        acorr_all_adj = []
        acorr_all_TS = []
        
        # Second Inner Loop over Sample Size
        for i in tqdm(range(N_Sim)):
            
            # Simulate Data
            if lam == 6.25:
            
                arima = ARIMA_est_gdpy.simulate(nsimulations=N_obs)
                
                param = "6_25"
            
            if lam == 1600:
                
                arima = ARIMA_est_gdpq.simulate(nsimulations=N_obs)
                
                param = "1600"
                
            if lam == 1562.25:
                
                arima = ARIMA_est_cgdpy.simulate(nsimulations=N_obs)
                
                param = "1562_25"
                
            if lam == 400000:
                
                arima = ARIMA_est_cgdpq.simulate(nsimulations=N_obs)
                
                param = "400000"
            
                
            # Pick Process
            us_series = np.cumsum(arima)
            
            # Two-Sided Filter
            cycle_TS, trend_TS = sm.tsa.filters.hpfilter(us_series[burn_in:], lam)
    
            filtered_cycles_OS = []
            filtered_cycles_adj = []
    
            # Expanding window filtering One Sided
            for k in range(0, len(us_series)-burn_in):
    
                cycle, trend = sm.tsa.filters.hpfilter(us_series[0:burn_in+k], lam)
    
                filtered_cycles_OS.append(np.asarray(cycle)[-1])
    
            # Expanding window filtering One Sided adjusted
            for k in range(0, len(us_series)-burn_in):
    
                cycle, trend = sm.tsa.filters.hpfilter(us_series[0:burn_in+k], lam_adj)
    
                filtered_cycles_adj.append(np.asarray(cycle)[-1])
            
            # Rescale cyclical Component
            filtered_cycles_adj = kap*np.asarray(filtered_cycles_adj)
            
            # Standard Deviation 
            # Store Autocorrelation for HP-1s, HP-1s* and HP-2s
            acorr_OS.append(np.mean(acf(filtered_cycles_OS, nlags=12)[1:]))
            acorr_adj.append(np.mean(acf(filtered_cycles_adj,  nlags=12)[1:]))
            acorr_TS.append(np.mean(acf(cycle_TS, nlags=12)[1:]))
            
            acorr_all_OS.append(acf(filtered_cycles_OS, nlags=12)[1:])
            acorr_all_adj.append(acf(filtered_cycles_adj, nlags=12)[1:])
            acorr_all_TS.append(acf(cycle_TS, nlags=12)[1:])
            
            # Store Standard Deviation for HP-1s, HP-1s* and HP-2s
            sd_OS.append(np.std(filtered_cycles_OS))
            sd_adj.append(np.std(filtered_cycles_adj))
            sd_TS.append(np.std(cycle_TS))
            
        
        sd_arima = pd.DataFrame({"One-Sided": sd_OS,
                                 "Adjusted": sd_adj,
                                 "Two-Sided": sd_TS})
        
     
        acorr_arima = pd.DataFrame({"One-Sided": acorr_OS,
                                    "Adjusted": acorr_adj,
                                    "Two-Sided": acorr_TS})
        
        acorr_all_OS = np.array(acorr_all_OS)
        acorr_all_TS = np.array(acorr_all_TS)
        acorr_all_adj = np.array(acorr_all_adj)
        
        
        sd = {"arima": sd_arima}
        
        acorr = {"arima": acorr_arima}
        
        acf_arrays = {"acf_OS_" + param + "_"  + str(N): acorr_all_OS,
                      "acf_TS_" + param + "_"  + str(N): acorr_all_TS,
                      "acf_adj_" + param + "_"  + str(N): acorr_all_adj}     
        
        sigma.append(sd)
        rho.append(acorr)
        acfs.append(acf_arrays)
        
    sigma_all.append(sigma)
    rho_all.append(rho)
    acf_all.append(acfs)

    

# Collect results in dictionaries (just for later use)
simulated_sigmas = {"50": sigma_all[0],
                    "100": sigma_all[1],
                    "500": sigma_all[2],
                    "1000": sigma_all[3]}
    
simulated_rhos = {"50": rho_all[0],
                  "100": rho_all[1],
                  "500": rho_all[2],
                  "1000": rho_all[3]}

simulated_acf = {"50": acf_all[0],
                 "100": acf_all[1],
                 "500": acf_all[2],
                 "1000": acf_all[3]}
   
 
'''
Print Rows 7 - 19 (ARIMA Simulations) for Columns 1 and 2 
(relative difference of standard deviations and absolute difference 
of autocorrelations) from Table 1 in the main paper. 
'''

# Print results for standard deviations
print("\n T=100: -------------- ARIMA (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 100, [6.25, 1600, 1562.25, 400000], "arima")


# Print results for autocorrelations
print("\n -------------- ARIMA (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 100, [6.25, 1600, 1562.25, 400000], "arima")


###############################################################################
# Results from the Online Appendix for Sample Sizes 50, 500, 1000

# Sample Size 50

print("\n T=50: \n -------------- ARIMA (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 50, [6.25, 1600, 1562.25, 400000], "arima")


# Print results for autocorrelations
print("\n -------------- ARIMA (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 50, [6.25, 1600, 1562.25, 400000], "arima")

###############################################################################

# Sample Size 500

print("\n T=500: \n -------------- ARIMA (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 500, [6.25, 1600, 1562.25, 400000], "arima")


# Print results for autocorrelations
print("\n -------------- ARIMA (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 500, [6.25, 1600, 1562.25, 400000], "arima")

###############################################################################

# Sample Size 1000

print("\n T=1000: \n -------------- ARIMA (Sigma) -------------- \n")
sry.tab_ratio(simulated_sigmas, 1000, [6.25, 1600, 1562.25, 400000], "arima")

# Print results for autocorrelations
print("\n -------------- ARIMA (Rho) -------------- \n")
sry.tab_difference(simulated_rhos, 1000, [6.25, 1600, 1562.25, 400000], "arima")


