# -*- coding: utf-8 -*-
"""
Created on Thu Oct 12 20:47:15 2023

@author: elias
"""

import numpy as np
import pandas as pd


'''
Code for the Simulations of the paper "On Adjusting the One-Sided HP-Filter" 
by Elias Wolf, Frieder Mokinsiki and Yves Schüler. This Code provides funcions
to create the summary statistics shown in Table 1. 

For further questions please contact: e.wolf@uni-bonn.de

'''


# Function to pick simulation results
def pick_sim(all_sim, sample_size, lam, process):
    
    N_sim_list = [50, 100, 500, 1000]
    
    sim_index = N_sim_list.index(sample_size)
           
    lam_index = [6.25, 1600, 1562.25, 400000].index(lam)

    results_df = all_sim[str(N_sim_list[sim_index])][lam_index]

    return(results_df[process])


# Function to calculate the mean of the relative difference 
def tab_ratio(all_sim, sample_size, lam_table, process):
    
    summary_stats = []
    
    for lam in lam_table:
    
        # Get simulation results
        df_out = pick_sim(all_sim, sample_size, lam, process).copy(deep=True)
        
        # Compute relative distance
        for i in range(2):
            
            df_out.iloc[:,i] = df_out.iloc[:,i]/df_out.iloc[:,2] - 1
        
        # Check fraction that is closer to HP-2s
        frac_adj = np.sum(np.abs(df_out.iloc[:,1]) < np.abs(df_out.iloc[:,0]))/df_out.shape[0]
        frac_os = 1 - frac_adj

        ratio_mean = df_out.mean(axis=0)  
    
        summary_stats.append([ratio_mean[1], frac_adj, ratio_mean[0], frac_os])
    
    summary_stats = np.array(summary_stats)
    
    table_out = pd.DataFrame({"HP-1s*": summary_stats[:,0],
                             "Frac_adj": summary_stats[:,1],
                             "HP-1s": summary_stats[:,2],
                             "Frac_OS": summary_stats[:,3],}, 
                             index=[str(lam) for lam in lam_table])
    
    print(table_out)

# Function to calculate the mean of the absolute difference     
def tab_difference(all_sim, sample_size, lam_table, process):
        
    summary_stats = []
    
    for lam in lam_table:
    
        # Get simulation results
        df_out = pick_sim(all_sim, sample_size, lam, process).copy(deep=True)
        
        # Compute relative distance
        for i in range(2):
            
            df_out.iloc[:,i] = df_out.iloc[:,i] - df_out.iloc[:,2]
        
        # Check fraction that is closer to HP-2s
        frac_adj = np.sum(np.abs(df_out.iloc[:,1]) < np.abs(df_out.iloc[:,0]))/df_out.shape[0]
        frac_os = 1 - frac_adj

        ratio_mean = df_out.mean(axis=0)  
    
        summary_stats.append([ratio_mean[1], frac_adj, ratio_mean[0], frac_os])
    
    summary_stats = np.array(summary_stats)
    
    table_out = pd.DataFrame({"HP-1s*": summary_stats[:,0],
                             "Frac_adj": summary_stats[:,1],
                             "HP-1s": summary_stats[:,2],
                             "Frac_OS": summary_stats[:,3],}, 
                             index=[str(lam) for lam in lam_table])
    
    print(table_out)